@@ -1,5 +1,5 @@
 # -*- Mode: cperl; cperl-indent-level: 4 -*-
-# $Id: Harness.pm,v 1.14.2.8 2001/09/29 04:06:23 schwern Exp $
+# $Id: Harness.pm,v 1.14.2.9 2001/12/16 04:54:30 schwern Exp $
 
 package Test::Harness;
 
@@ -22,10 +22,15 @@ use vars qw($VERSION $Verbose $Switches $Have_Devel_Corestack $Curtest
 
 $Have_Devel_Corestack = 0;
 
-$VERSION = 2.00_01;
+$VERSION = 2.00_03;
 
 $ENV{HARNESS_ACTIVE} = 1;
 
+END {
+    # For VMS.
+    delete $ENV{HARNESS_ACTIVE};
+}
+
 # Some experimental versions of OS/2 build have broken $?
 my $Ignore_Exitcode = $ENV{HARNESS_IGNORE_EXITCODE};
 
@@ -435,22 +440,7 @@ sub _run_all_tests {
                 bench    => 0,
                );
 
-    # pass -I flags to children
-    my $old5lib = $ENV{PERL5LIB};
-
-    # VMS has a 255-byte limit on the length of %ENV entries, so
-    # toss the ones that involve perl_root, the install location
-    # for VMS
-    my $new5lib;
-    if ($^O eq 'VMS') {
-        $new5lib = join($Config{path_sep}, grep {!/perl_root/i;} @INC);
-        $Switches =~ s/-(\S*[A-Z]\S*)/"-$1"/g;
-    }
-    else {
-        $new5lib = join($Config{path_sep}, @INC);
-    }
-
-    local($ENV{'PERL5LIB'}) = $new5lib;
+    local($ENV{'PERL5LIB'}) = $Strap->_INC2PERL5LIB;
 
     my @dir_files = _globdir $Files_In_Dir if defined $Files_In_Dir;
     my $t_start = new Benchmark;
@@ -583,13 +573,7 @@ sub _run_all_tests {
     }
     $tot{bench} = timediff(new Benchmark, $t_start);
 
-    if ($^O eq 'VMS') {
-        if (defined $old5lib) {
-            $ENV{PERL5LIB} = $old5lib;
-        } else {
-            delete $ENV{PERL5LIB};
-        }
-    }
+    $Strap->_restore_PERL5LIB;
 
     return(\%tot, \%failedtests);
 }
@@ -737,12 +721,6 @@ sub _open_test {
     }
 }
 
-sub _run_one_test {
-    my($test) = @_;
-
-    
-}
-
 
 sub _parse_test_line {
     my($line, $test, $tot) = @_;
@@ -865,16 +843,8 @@ sub _close_fh {
 sub _set_switches {
     my($test) = shift;
 
-    local *TEST;
-    open(TEST, $test) or print "can't open $test. $!\n";
-    my $first = <TEST>;
     my $s = $Switches;
-    $s .= " $ENV{'HARNESS_PERL_SWITCHES'}"
-      if exists $ENV{'HARNESS_PERL_SWITCHES'};
-    $s .= join " ", q[ "-T"], map {qq["-I$_"]} @INC
-      if $first =~ /^#!.*\bperl.*-\w*T/;
-
-    close(TEST) or print "can't close $test. $!\n";
+    $s .= $Strap->_switches($test);
 
     return $s;
 }
