@@ -1,11 +1,12 @@
 # -*- Mode: cperl; cperl-indent-level: 4 -*-
-# $Id: Straps.pm,v 1.1.2.6 2001/09/29 03:53:03 schwern Exp $
+# $Id: Straps.pm,v 1.1.2.8 2001/12/16 05:16:34 schwern Exp $
 
 package Test::Harness::Straps;
 
 use strict;
 use vars qw($VERSION);
-$VERSION = '0.02';
+use Config;
+$VERSION = '0.03';
 
 use Test::Harness::Assert;
 
@@ -85,6 +86,7 @@ Initialize the internal state of a strap to make it ready for parsing.
 sub _init {
     my($self) = shift;
 
+    $self->{_is_vms} = $^O eq 'VMS';
 }
 
 
@@ -118,7 +120,7 @@ sub analyze {
                    skip     => 0,
                    bonus    => 0,
                    
-                   summary  => []
+                   details  => []
                   );
 
 
@@ -167,8 +169,8 @@ sub analyze {
 
             $totals{ok}++ if $pass;
 
-            $totals{summary}[$result{number}] = 
-              {$self->_summarize($pass, \%result)};
+            $totals{details}[$result{number} - 1] = 
+              {$self->_detailize($pass, \%result)};
 
             $self->{'next'}++;
 
@@ -210,20 +212,107 @@ sub analyze_file {
 
     local $ENV{IFS} = '';
 
-    # If $^X is just 'perl' we'll have to figure it out from PATH.
-    # So rather than implicitly trusting PATH, we'll just save
-    # ourselves a lot of work and explicitly trust it.
-    local($ENV{PATH}) = $ENV{PATH} =~ /(.*)/;
+    # XXX set switches
 
-    unless( open(FILE, '-|') || exec $^X, $file ) {
+    local $ENV{PERL5LIB} = $self->_INC2PERL5LIB;
+
+    # Is this necessary anymore?
+    my $cmd = $self->{_is_vms} ? "MCR $^X" : $^X;
+
+    my $switches = $self->_switches($file);
+
+    # *sigh* this breaks under taint, but open -| is unportable.
+    unless( open(FILE, "$cmd $switches $file|") ) {
         # XXX fix up this error handling.
-        die $!;
+        print "can't run $file. $!\n";
+        return;
     }
 
     # XXX simple, stupid, easy for now.
-    $self->analyze_fh($file, \*FILE);
+    my %results = $self->analyze_fh($file, \*FILE);
+
+    $self->_restore_PERL5LIB();
+
+    return %results;
 }
 
+=begin _private
+
+=item B<_switches>
+
+  my $switches = $self->_switches($file);
+
+Formats and returns the switches necessary to run the test.
+
+=cut
+
+sub _switches {
+    my($self, $file) = @_;
+
+    local *TEST;
+    open(TEST, $file) or print "can't open $file. $!\n";
+    my $first = <TEST>;
+    my $s = '';
+    $s .= " $ENV{'HARNESS_PERL_SWITCHES'}"
+      if exists $ENV{'HARNESS_PERL_SWITCHES'};
+    $s .= join " ", qq[ "-$1"], map {qq["-I$_"]} @INC
+      if $first =~ /^#!.*\bperl.*-\w*([Tt]+)/;
+
+    close(TEST) or print "can't close $file. $!\n";
+
+    return $s;
+}
+
+
+=item B<_INC2PERL5LIB>
+
+  local $ENV{PERL5LIB} = $self->_INC2PERL5LIB;
+
+Takes the current value of @INC and turns it into something suitable
+for putting onto PERL5LIB.
+
+=cut
+
+sub _INC2PERL5LIB {
+    my($self) = shift;
+
+    $self->{_old5lib} = $ENV{PERL5LIB};
+
+    # VMS has a 255-byte limit on the length of %ENV entries, so
+    # toss the ones that involve perl_root, the install location
+    # for VMS
+    my $new5lib;
+    if ($self->{_is_vms}) {
+        $new5lib = join($Config{path_sep}, grep {!/perl_root/i;} @INC);
+    }
+    else {
+        $new5lib = join($Config{path_sep}, @INC);
+    }
+
+    return $new5lib;
+}    
+
+=item B<_restore_PERL5LIB>
+
+  $self->_restore_PERL5LIB;
+
+This restores the original value of the PERL5LIB environment variable.
+Necessary on VMS, otherwise a no-op.
+
+=cut
+
+sub _restore_PERL5LIB {
+    my($self) = shift;
+
+    return unless $self->{_is_vms};
+
+    if (defined $self->{_old5lib}) {
+        $ENV{PERL5LIB} = $self->{_old5lib};
+    }
+}
+    
+
+=end _private
 
 =back
 
@@ -276,7 +365,7 @@ REGEX
 sub _is_header {
     my($self, $line) = @_;
 
-    if( my($max, $extra) = $line =~ /^1..(\d+)(.*)/ ) {
+    if( my($max, $extra) = $line =~ /^1\.\.(\d+)(.*)/ ) {
         $self->{max}  = $max;
         assert( $self->{max} >= 0,  'Max # of tests looks right' );
 
@@ -336,9 +425,9 @@ sub _is_test {
         my($name, $control) = split /(?:[^\\]|^)#/, $extra if $extra;
         my($type, $reason)  = $control =~ /^\s*(\S+)(?:\s+(.*))?$/ if $control;
 
-        $test->{number} = $num;
+        $test->{number} = $num || '';
         $test->{ok}     = $not ? 0 : 1;
-        $test->{name}   = $name;
+        $test->{name}   = $name || '';
 
         if( defined $type ) {
             $test->{type}   = $type =~ /^TODO$/i ? 'todo' :
@@ -347,7 +436,7 @@ sub _is_test {
         else {
             $test->{type} = '';
         }
-        $test->{reason} = $reason if $reason;
+        $test->{reason} = $reason ? $reason : '';
 
         return $YES;
     }
@@ -427,12 +516,12 @@ The %results returned from analyze() contain the following information:
 So a successful test should have max == seen == ok.
 
 
-There is one final item, the summary.
+There is one final item, the details.
 
-  summary           an array ref reporting the result of 
+  details           an array ref reporting the result of 
                     each test looks like this:
 
-    $results{summary}[$test_num] = 
+    $results{details}[$test_num - 1] = 
             { ok        => is the test considered ok?
               actual_ok => did it literally say 'ok'?
               name      => name of the test (if any)
@@ -440,54 +529,51 @@ There is one final item, the summary.
               reason    => reason for the above (if any)
             };
 
-  Element 0 is unused (but reserved for future use).
+Element 0 of the details is test #1.  I tried it with element 1 being
+#1 and 0 being empty, this is less awkward.
 
 =begin _private
 
 =over 4
 
-=item B<_summarize>
+=item B<_detailize>
 
-  my %summary = $strap->_summarize($pass, \%test);
+  my %details = $strap->_detailize($pass, \%test);
 
-Generates the summary based on the last test line seen.  $pass is true
+Generates the details based on the last test line seen.  $pass is true
 if it was considered to be a passed test.  %test is the results of the
 test you're summarizing.
 
 =cut
 
-sub _summarize {
+sub _detailize {
     my($self, $pass, $test) = @_;
 
-    my %summary = ( ok         => $pass,
+    my %details = ( ok         => $pass,
                     actual_ok  => $test->{ok}
                   );
 
-    assert( !(grep !defined $summary{$_}, keys %summary),
+    assert( !(grep !defined $details{$_}, keys %details),
             'test contains the ok and actual_ok info' );
 
     foreach my $piece (qw(name type reason)) {
-        $summary{$piece} = $test->{$piece} if $test->{$piece};
+        $details{$piece} = $test->{$piece} if $test->{$piece};
     }
 
-    return %summary;
+    return %details;
 }
 
 =end _private
 
 =back
 
-=head1 NOTES
-
-Test::Harness::Straps will run fine under taint mode.
-
 =head1 AUTHOR
 
 Michael G Schwern E<lt>schwern@pobox.comE<gt>
 
 =head1 SEE ALSO
 
-L<Test::Harness>, L<Test::Harness::Assert>
+L<Test::Harness>
 
 =cut
 
